@@ -2,11 +2,11 @@ use 5.008;
 use strict;
 use warnings;
 package BenchmarkAnything::Storage::Frontend::Lib;
-# git description: v0.016-3-g9ff088e
+# git description: v0.017-7-g7551722
 
 our $AUTHORITY = 'cpan:SCHWIGON';
 # ABSTRACT: Basic functions to access a BenchmarkAnything store
-$BenchmarkAnything::Storage::Frontend::Lib::VERSION = '0.017';
+$BenchmarkAnything::Storage::Frontend::Lib::VERSION = '0.018';
 use Scalar::Util 'reftype';
 
 
@@ -196,9 +196,16 @@ sub connect
                 my $dbh      = DBI->connect($dsn, $user, $password, {'RaiseError' => 1})
                  or die "benchmarkanything: can not connect: ".$DBI::errstr;
 
+                # external search engine
+                my $searchengine = $self->{config}{benchmarkanything}{searchengine} || {};
+
                 # remember
                 $self->{dbh}     = $dbh;
-                $self->{backend} = BenchmarkAnything::Storage::Backend::SQL->new({dbh => $dbh, debug => $self->{debug} });
+                $self->{backend} = BenchmarkAnything::Storage::Backend::SQL->new({dbh => $dbh,
+                                                                                  debug => $self->{debug},
+                                                                                  force => $self->{force},
+                                                                                  (keys %$searchengine ? (searchengine => $searchengine) : ()),
+                                                                                 });
         }
         elsif ($backend eq 'http')
         {
@@ -683,12 +690,47 @@ sub process_raw_result_queue
         }
         else
         {
-                die "benchmarkanything: only backend 'local' allowed here.\n";
+                die "benchmarkanything: only backend 'local' allowed in 'process_raw_result_queue'.\n";
         }
         return;
 }
 
 
+sub init_search_engine
+{
+        my ($self, $force) = @_;
+
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
+        {
+                $self->{backend}->init_search_engine($force);
+        }
+        else
+        {
+                die "benchmarkanything: only backend 'local' allowed in 'init_search_engine'.\n";
+        }
+        return;
+}
+
+
+sub sync_search_engine
+{
+        my ($self, $force, $start, $count) = @_;
+
+        my $backend = $self->{config}{benchmarkanything}{backend};
+        if ($backend eq 'local')
+        {
+                $self->{backend}->sync_search_engine($force, $start, $count);
+        }
+        else
+        {
+                die "benchmarkanything: only backend 'local' allowed in 'sync_search_engine'.\n";
+        }
+        return;
+}
+
+
+
 sub getpoint
 {
         my ($self, $value_id) = @_;
@@ -959,6 +1001,45 @@ Works on the queued entries created by C<add> in I<queuemode=1>. It
 finishes as soon as there are no more unprocessed raw entries, or it
 processed C<$count> entries (default=10).
 
+=head2 init_search_engine($force)
+
+Initializes the configured search engine (Elasticsearch). If the index
+already exists it does nothing, except when you set C<$force> to a
+true value which deletes and re-creates the index. This is necessary
+for example to apply new type mappings.
+
+After a successful (re-)init you need to run C<sync_search_engine>.
+
+During (re-init) and sync you should disable querying by setting
+
+  benchmarkanything.searchengine.elasticsearch.enable_query: 0
+
+=head3 Options
+
+=over 4
+
+=item force
+
+If set, an existing index is deleted before (re-)creating.
+
+=back
+
+=head2 sync_search_engine($force, $start, $count)
+
+Synchronizes entries from the ::SQL backend into the configured search
+engine (usually Elasticsearch). It starts at entry C<$start> and bulk
+indexes in blocks of C<$count>.
+
+=head3 Options
+
+=over 4
+
+=item force
+
+If set, all entries are (re-)indexed, not just the new ones.
+
+=back
+
 =head2 getpoint ($value_id)
 
 Returns a single benchmark point with B<all> its key/value pairs.
@@ -969,7 +1050,7 @@ Steffen Schwigon <ss5@renormalist.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2016 by Steffen Schwigon.
+This software is copyright (c) 2017 by Steffen Schwigon.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
